
#include "lcd.h";


//-------------------------- LCD ----------------------------------//
void LcdInit();
void LcdDataOut();
void LcdPutCGRAM(byte NrZnaku);
void LcdSendCommand(unsigned char command);
void LcdSendNible(unsigned char command);
void LcdSendByte(unsigned char command);
void LcdSendData(unsigned char data);
void LcdSetCursorPosition(unsigned char row,unsigned char line);
void LcdPrint(char* text);
void LcdPrint_P(char* text);
void LcdPrintChar(char character);
char LcdWin2Lcd(char znak);
void LcdNextLine();
void LcdClr();
#define	LcdCls()		LcdClr()
#define	LcdDispayOn()	LcdSendCommand(0xC)
#define	LcdDisplayOff()	LcdSendCommand(0x8)
#define	LcdCursorOn()  	LcdSendCommand(0xF)	// 0xE bez migania
#define	LcdCursorOff() 	LcdSendCommand(0xC)
#define	LcdHome()  		LcdSendCommand(0x2); lcdv=0; lcdh=0
uchar lcdv=0,lcdh=0;

unsigned char LCDLINE = 2;		// Liczba wierszy
unsigned char LCDROW = 16;		// Liczba kolumn


//-----------------------------------------------------------------
//			Init LCD
//-----------------------------------------------------------------
void LcdInit()
{
#ifdef	CHAR_PL
  byte x;
#endif

 	LcdRSddr;								// Porty wyjciami
	LcdEddr;
	LcdEclr;								// E=0
	LcdDataOut();
	LcdRSclr;								// RS=0

   	_delay_ms(20);
	LcdSendNible(0x03); 					// Init magistrali 8-bit
  	_delay_ms(6);
	LcdSendNible(0x03); 					// Init magistrali 8-bit
 	_delay_ms(1);
	LcdSendNible(0x03); 					// Init magistrali 8-bit
 	_delay_ms(1);
	#ifdef LCD8BIT
  	 LcdSendCommand(0x38); 					// FunctionSet 	%0011_1000	(0  0  1 DL  N  F  -  -) DL=8bit, N=1 2lines, F=1 5*10dots
	#else
	 LcdSendNible(0x02); 					// Init magistrali 8-bit
 	 _delay_ms(1);
  	 LcdSendCommand(0x28); 					// FunctionSet 	%0010_1000	(0  0  1 DL  N  F  -  -) DL=4bit, N=1 2lines, F=1 5*10dots
	#endif
  	LcdSendCommand(0x06);					// EntryModeSet	%0000_0110	(0  0  0  0  0  1 ID  S) ID=1 increment, S=1
  	LcdSendCommand(0x0C);					// DisplayOnOff	%0000_1100	(0  0  0  0  1  D  C  B) D-on/off, C-cursor on/of, B-blinking character
  	LcdSendCommand(0x01);  					// DisplayClr	%0000_0001	(0  0  0  0  0  0  0  1)


	LcdDispayOn();

	//--- wysanie wzorow polskich znakw ---
#ifdef	CHAR_PL
	LcdSendCommand(0x40);					// Ustawienie na pocztek generatora znakw
	for	(x=0; x<64; x++)					// Liczba bajtw generatora
		{
		LcdSendData( pgm_read_byte( &znaki_pl[x]) );
		}
#endif

	LcdHome();
}




//-----------------------------------------------------------------
//			Ustawienie linii danych jako wyjcia
//-----------------------------------------------------------------
void LcdDataOut()
{
	#ifdef LCD8BIT
	 LcdD0ddr; LcdD1ddr; LcdD2ddr; LcdD3ddr;
	#endif
	LcdD4ddr; LcdD5ddr; LcdD6ddr; LcdD7ddr;
}



//-----------------------------------------------------------------
void LcdClr()
{
	 LcdSendCommand(0x1);
	 lcdv=0; lcdh=0;
}



//-----------------------------------------------------------------
void LcdSendCommand(unsigned char data)
{
	LcdDataOut();
  	LcdRSclr;								// RS=0;
	#ifdef LCD8BIT
	 LcdSendByte( data );
	#else
	 LcdSendNible( data >> 4 ); 	 		// Starsze 4 bity
 	 LcdSendNible( data ); 		 			// Modsze 4 bity
	#endif
	//--- Oczekiwanie na zakoczenie operacji ---
	if ( (data & 2) == 2 ) { _delay_ms(2); }		// LcdHome 1,5ms
	else if ( (data & 2) == 2 ) { _delay_ms(2); }	// Clear Display ?,?ms
	_delay_us(40);									// Pozostae rozkazy 37us
}



//-----------------------------------------------------------------
void LcdSendNible(unsigned char data)
{
//#define	DELAYLCD	180					// 180us juz jest ok na tamie 15cm
#define	DELAYLCD	200						// 250us jest gdy na D0..D3 jest klawiatura



//	LcdD0clr; LcdD1clr; LcdD2clr; LcdD3clr;	// Nieuywane linie do masy
	if ((data & 1) != 0) {LcdD4set;} else LcdD4clr;
	if ((data & 2) != 0) {LcdD5set;} else LcdD5clr;
	if ((data & 4) != 0) {LcdD6set;} else LcdD6clr;
	if ((data & 8) != 0) {LcdD7set;} else LcdD7clr;
 	_delay_us(DELAYLCD);
	LcdEset;								// E=1
	_delay_us(DELAYLCD);
	LcdEclr;								// E=0
	_delay_us(DELAYLCD);
}



//-----------------------------------------------------------------
#ifdef LCD8BIT
void LcdSendByte(unsigned char data)
{

	if ((data & 0x01) != 0) {LcdD0set;} else LcdD0clr;
	if ((data & 0x02) != 0) {LcdD1set;} else LcdD1clr;
	if ((data & 0x04) != 0) {LcdD2set;} else LcdD2clr;
	if ((data & 0x08) != 0) {LcdD3set;} else LcdD3clr;
	if ((data & 0x10) != 0) {LcdD4set;} else LcdD4clr;
	if ((data & 0x20) != 0) {LcdD5set;} else LcdD5clr;
	if ((data & 0x40) != 0) {LcdD6set;} else LcdD6clr;
	if ((data & 0x80) != 0) {LcdD7set;} else LcdD7clr;
 	_delay_us(DELAYLCD);
	LcdEset;								// E=1
	_delay_us(DELAYLCD);
	LcdEclr;								// E=0
	_delay_us(DELAYLCD);
}
#endif


//-----------------------------------------------------------------
void LcdSendData(unsigned char data)
{
	LcdDataOut();
	LcdRSset;								// RS=1
	#ifdef LCD8BIT
	 LcdSendByte( data );
	#else
	 LcdSendNible( data >> 4 );  			// Starsze 4 bity
	 LcdSendNible( data ); 		 			// Modsze 4 bity
	#endif
}



//-----------------------------------------------------------------
void LcdNextLine()
{
	lcdh = 0;
	if ( (++lcdv) >= LCDLINE) lcdv=0;
	LcdSetCursorPosition(lcdh,lcdv);
}



//-----------------------------------------------------------------
void LcdPrintChar(char character)
{
	LcdSendData(character);


	if ( (++lcdh) >= LCDROW)			// Po zapisaniu znaku
		{								// Jeli przektroczono dugo wiersza to:
		if ( (++lcdv) >= LCDLINE)		// Nastpny wiersz, jeli ostatni to:
			{
			LcdHome();					// Wracamy na poczatek
			}
		switch(lcdv)
			{
			case(0): LcdSetCursorPosition(0,0); break;
			case(1): LcdSetCursorPosition(0,1); break;
			case(2): LcdSetCursorPosition(0,2); break;
			case(3): LcdSetCursorPosition(0,3); break;
			}
		}
}



//-----------------------------------------------------------------
void LcdSetCursorPosition(unsigned char row,unsigned char line)
{
	lcdh = row;
	lcdv = line;
	if (LCDLINE == 2)
		{
  		if(line==1) row += 0x40;		// Pawdziwe dla wyswietlaczy 2 * x
		}
	if (LCDLINE == 4)
		{
  		if(line==1) row += 0x40;		// Pawdziwe dla wyswietlaczy 4 * x
  		if(line==2) row += LCDROW;		// Pawdziwe dla wyswietlaczy 4 * x
  		if(line==3) row += 0x40+LCDROW;	// Pawdziwe dla wyswietlaczy 4 * x
		}
  	row |= 0x80;						// Kod rozkazu ($80) OR pozycja w RAM LCD
  	LcdSendCommand(row);
}



//-----------------------------------------------------------------
void LcdPrint(char* text)
{
char znak;

	while ( (znak=*text++) )					// Wczytaj znak z RAM do zmiennej i sprawdz czy rozna od 0
  	 	{
		LcdPrintChar( LcdWin2Lcd(znak) );		// Skonwertowany znak daj na LCD
		}
}


//-----------------------------------------------------------------
void LcdPrint_P(char* text)
{
char znak;

	while ( (znak=pgm_read_byte(text++)) )		// Wczytaj znak z FLASH do zmiennej i sprawdz czy rozna od 0
  	 	{
		LcdPrintChar( LcdWin2Lcd(znak) );		// Skonwertowany znak daj na LCD
		}
}


//-----------------------------------------------------------------
char LcdWin2Lcd(char znak)
{
#ifdef	CHAR_PL
	switch(znak)
		{
		case(''): return(0);
		case(''): return(1);
		case(''): return(2);
		case(''): return(3);
		case(''): return(4);
		case(''): return(5);
		case(''): return(6);
		case(''): return(7);
		case(''): return(7);
		case(''): return(0);
		case(''): return(1);
		case(''): return(2);
		case(''): return(3);
		case(''): return(4);
		case(''): return(5);
		case(''): return(6);
		case(''): return(7);
		case(''): return(7);
		}
#endif
	return(znak);
}



//-----------------------------------------------------------------
#ifdef	CHAR_PL
void LcdPutCGRAM(byte NrZnaku)
{
byte x;


	LcdDataOut();
	//--- wysanie wzoru znaku ---

	LcdSendCommand(0x40 + (NrZnaku<<3) );	// Ustawienie generatora znakw
	for	(x=0; x<8; x++)						// Liczba bajtw znaku
		{
		LcdSendData( pgm_read_byte( &znaki_pl[x]) );
		}

}
#endif



